import Plotly from 'plotly.js-dist';

export function renderMeasurementResults(measurementResults, language) {
  console.debug('renderMeasurementResults called with:', measurementResults);
  const container = document.getElementById('measurementResults');
  container.innerHTML = '';

  if (!measurementResults) {
    console.error('No measurementResults provided.');
    return;
  }

  let results = measurementResults['dcc:measurementResult'];
  console.debug('Raw dcc:measurementResult:', results);

  if (!results) {
    console.error("Missing 'dcc:measurementResult' in measurementResults");
    return;
  }

  if (!Array.isArray(results)) {
    results = [results];
  }
  console.debug('Processed measurement result array:', results);

  // Use the first measurementResult object
  const measurementResult = results[0];
  console.debug('Using measurementResult:', measurementResult);

  // Extract the result object from dcc:results -> dcc:result
  let resultObj = measurementResult['dcc:results'] && measurementResult['dcc:results']['dcc:result'];
  if (!resultObj) {
    console.error("Missing 'dcc:results' or 'dcc:result' in measurementResult:", measurementResult);
    return;
  }
  if (Array.isArray(resultObj)) {
    resultObj = resultObj[0];
  }
  console.debug('Using result object:', resultObj);

  // Get the measurement result name from dcc:name (from measurementResult)
  let resultName = 'Measurement Result';
  if (measurementResult['dcc:name'] && measurementResult['dcc:name']['dcc:content']) {
    let content = measurementResult['dcc:name']['dcc:content'];
    if (Array.isArray(content)) {
      const match = content.find(item => item.$ && item.$.lang === language) || content[0];
      resultName = match._ || match;
    } else {
      resultName = content._ || content;
    }
  }
  console.debug('Result name:', resultName);

  const tabTitle = document.createElement('h2');
  tabTitle.textContent = resultName + ' (' + language + ')';
  container.appendChild(tabTitle);

  // Now extract dcc:data -> dcc:list from the resultObj
  if (!resultObj['dcc:data'] || !resultObj['dcc:data']['dcc:list']) {
    console.error("Missing 'dcc:data' or 'dcc:list' in result object:", resultObj);
    return;
  }
  const listData = resultObj['dcc:data']['dcc:list'];
  console.debug('List data:', listData);

  // Flatten all quantities from the list
  let quantities = [];
  if (listData['dcc:quantity']) {
    quantities = Array.isArray(listData['dcc:quantity']) ? listData['dcc:quantity'] : [listData['dcc:quantity']];
  }
  console.debug('Quantities from list:', quantities);

  // Also add quantities from measurementMetaData if available
  if (listData['dcc:measurementMetaData'] && listData['dcc:measurementMetaData']['dcc:metaData']) {
    let metaData = listData['dcc:measurementMetaData']['dcc:metaData'];
    if (!Array.isArray(metaData)) metaData = [metaData];
    metaData.forEach(md => {
      if (md['dcc:data'] && md['dcc:data']['dcc:quantity']) {
        let qs = md['dcc:data']['dcc:quantity'];
        if (!Array.isArray(qs)) qs = [qs];
        quantities = quantities.concat(qs);
      }
    });
  }
  console.debug('Combined quantities:', quantities);

  // Separate index quantities and data quantities
  const indexQuantities = [];
  const dataQuantities = [];
  quantities.forEach(q => {
    if (q.$ && q.$.refType && q.$.refType.match(/basic_tableIndex/)) {
      indexQuantities.push(q);
    } else {
      dataQuantities.push(q);
    }
  });
  console.debug('Index Quantities:', indexQuantities);
  console.debug('Data Quantities:', dataQuantities);

  // Create radio buttons for X-axis selection (from indexQuantities)
  const xAxisContainer = document.createElement('div');
  xAxisContainer.innerHTML = '<strong>Select X-Axis:</strong> ';
  indexQuantities.forEach((q, idx) => {
    let nameStr = 'Index ' + idx;
    if (q['dcc:name'] && q['dcc:name']['dcc:content']) {
      let content = q['dcc:name']['dcc:content'];
      if (Array.isArray(content)) {
        const match = content.find(item => item.$ && item.$.lang === language) || content[0];
        nameStr = match._ || match;
      } else {
        nameStr = content._ || content;
      }
    }
    const radio = document.createElement('input');
    radio.type = 'radio';
    radio.name = 'xAxisSelect';
    radio.value = idx;
    if (idx === 0) radio.checked = true;
    const label = document.createElement('label');
    label.textContent = nameStr;
    label.style.marginRight = '10px';
    xAxisContainer.appendChild(radio);
    xAxisContainer.appendChild(label);
  });
  container.appendChild(xAxisContainer);

  // Tolerance toggle (placeholder)
  const toleranceToggle = document.createElement('input');
  toleranceToggle.type = 'checkbox';
  toleranceToggle.id = 'toleranceToggle';
  const tolLabel = document.createElement('label');
  tolLabel.htmlFor = 'toleranceToggle';
  tolLabel.textContent = ' Enable tolerance markings';
  const tolContainer = document.createElement('div');
  tolContainer.appendChild(toleranceToggle);
  tolContainer.appendChild(tolLabel);
  container.appendChild(tolContainer);

  // Create containers for plots and table
  const plotsContainer = document.createElement('div');
  plotsContainer.id = 'plotsContainer';
  container.appendChild(plotsContainer);
  const tableContainer = document.createElement('div');
  tableContainer.id = 'tableContainer';
  container.appendChild(tableContainer);

  // Function to update the visualization
  function updateVisualization() {
    const selectedRadio = document.querySelector('input[name="xAxisSelect"]:checked');
    if (!selectedRadio) {
      console.error('No X-Axis selection found.');
      return;
    }
    const selectedIndex = selectedRadio.value;
    const xQuantity = indexQuantities[selectedIndex];
    let xValues = [];
    if (xQuantity && xQuantity['si:realListXMLList'] && xQuantity['si:realListXMLList']['si:valueXMLList']) {
      xValues = xQuantity['si:realListXMLList']['si:valueXMLList'].trim().split(/\s+/).map(v => parseFloat(v));
    }
    console.debug('Selected X-Axis values:', xValues);

    // Build table headers and values
    const headers = [];
    let xHeader = 'X-Axis';
    if (xQuantity['dcc:name'] && xQuantity['dcc:name']['dcc:content']) {
      let content = xQuantity['dcc:name']['dcc:content'];
      if (Array.isArray(content)) {
        const match = content.find(item => item.$ && item.$.lang === language) || content[0];
        xHeader = match._ || match;
      } else {
        xHeader = content._ || content;
      }
    }
    headers.push(xHeader);

    const dataValues = [];
    dataQuantities.forEach(q => {
      let header = 'Data';
      if (q['dcc:name'] && q['dcc:name']['dcc:content']) {
        let content = q['dcc:name']['dcc:content'];
        if (Array.isArray(content)) {
          const match = content.find(item => item.$ && item.$.lang === language) || content[0];
          header = match._ || match;
        } else {
          header = content._ || content;
        }
      }
      headers.push(header);
      let values = [];
      if (q['si:realListXMLList'] && q['si:realListXMLList']['si:valueXMLList']) {
        values = q['si:realListXMLList']['si:valueXMLList'].trim().split(/\s+/).map(v => parseFloat(v));
      }
      if (values.length === 1 && xValues.length > 1) {
        values = new Array(xValues.length).fill(values[0]);
      }
      dataValues.push(values);
    });

    // Build table rows
    const tableData = [headers];
    for (let i = 0; i < xValues.length; i++) {
      const row = [];
      row.push(xValues[i]);
      dataValues.forEach(values => {
        row.push(values[i] !== undefined ? values[i] : '');
      });
      tableData.push(row);
    }
    console.debug('Table data:', tableData);
    renderTable(tableData);

    // Group data quantities by unit for plotting
    const unitGroups = {};
    dataQuantities.forEach((q, idx) => {
      let unit = '';
      if (q['si:realListXMLList'] && q['si:realListXMLList']['si:unitXMLList']) {
        unit = q['si:realListXMLList']['si:unitXMLList'].trim();
      }
      if (!unitGroups[unit]) {
        unitGroups[unit] = [];
      }
      let header = headers[idx + 1];
      let values = [];
      if (q['si:realListXMLList'] && q['si:realListXMLList']['si:valueXMLList']) {
        values = q['si:realListXMLList']['si:valueXMLList'].trim().split(/\s+/).map(v => parseFloat(v));
      }
      if (values.length === 1 && xValues.length > 1) {
        values = new Array(xValues.length).fill(values[0]);
      }
      unitGroups[unit].push({ name: header, y: values });
    });
    console.debug('Unit groups for plots:', unitGroups);

    // Clear and render plots
    plotsContainer.innerHTML = '';
    Object.keys(unitGroups).forEach(unit => {
      const graphDiv = document.createElement('div');
      graphDiv.style.width = '100%';
      graphDiv.style.height = '300px';
      plotsContainer.appendChild(graphDiv);

      const traces = unitGroups[unit].map(trace => {
        return {
          x: xValues,
          y: trace.y,
          type: 'scatter',
          mode: 'lines+markers',
          name: trace.name
        };
      });
      const layout = {
        title: 'Plot (' + unit + ')',
        xaxis: { title: xHeader },
        yaxis: { title: unit },
        hovermode: 'closest'
      };
      Plotly.newPlot(graphDiv, traces, layout).then(() => {
        console.debug('Plot rendered for unit:', unit);
      });

      // Coupled mouseover events for table row highlighting
      graphDiv.on('plotly_hover', function(data) {
        if (data.points && data.points.length > 0) {
          const pointIndex = data.points[0].pointIndex + 1; // offset for header row
          highlightTableRow(pointIndex);
        }
      });
      graphDiv.on('plotly_unhover', function() {
        clearTableRowHighlights();
      });
    });
  }

  // Render table from a 2D array
  function renderTable(tableData) {
    tableContainer.innerHTML = '';
    const table = document.createElement('table');
    tableData.forEach((rowData, rowIndex) => {
      const tr = document.createElement('tr');
      rowData.forEach(cellData => {
        const cell = document.createElement(rowIndex === 0 ? 'th' : 'td');
        cell.textContent = cellData;
        cell.style.padding = '4px';
        cell.style.border = '1px solid #ccc';
        tr.appendChild(cell);
      });
      tr.addEventListener('mouseover', () => { tr.style.backgroundColor = '#eef'; });
      tr.addEventListener('mouseout', () => { tr.style.backgroundColor = ''; });
      table.appendChild(tr);
    });
    tableContainer.appendChild(table);
  }

  // Functions for coupled table row highlights
  function highlightTableRow(rowIndex) {
    const rows = tableContainer.querySelectorAll('tr');
    if (rows[rowIndex]) {
      rows[rowIndex].style.backgroundColor = '#fee';
    }
  }
  function clearTableRowHighlights() {
    const rows = tableContainer.querySelectorAll('tr');
    rows.forEach(row => row.style.backgroundColor = '');
  }

  // Initial update
  updateVisualization();

  // Update visualization when X-axis selection changes
  const radios = document.querySelectorAll('input[name="xAxisSelect"]');
  radios.forEach(radio => {
    radio.addEventListener('change', updateVisualization);
  });

  // Tolerance toggle event (placeholder)
  toleranceToggle.addEventListener('change', () => {
    console.log('Tolerance toggle:', toleranceToggle.checked);
    // Future: update plot/table for tolerance markings and color coding
  });
}
